/**
 * $Id: driver.c 22 2006-05-03 20:12:20Z alexeen $
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <err.h>

#include "mh_key.h"
#include "minheap.h"

/** Private functions. **/
int insert(minheap_t *h, long int i, const char *data);
void get_size(minheap_t *h);
void extract_min(minheap_t *h);
void print_heap(minheap_t *h);
mh_key_t *gen_key(long int i);

/**
 * Test driver's main.
 */
int main(int argc, char **argv)
{
	minheap_t *my_heap = NULL;

	printf("<<< Test Driver for min-heap implementation! >>>\n\n");

	/** Create min-heap. **/
	printf("Creating a new min-heap ... ");
	my_heap = minheap_init();
	printf("done.\n\n");

	/** Do some insertions. **/
	printf("========== Insertions ==========\n\n");

	printf("Insert `1 : one' ... ");
	insert(my_heap, 1, "one");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `12 : twelve' ... ");
	insert(my_heap, 12, "twelve");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `2 : two' ... ");
	insert(my_heap, 2, "two");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `20 : twenty' ... ");
	insert(my_heap, 20, "twenty");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `5 : five' ... ");
	insert(my_heap, 5, "five");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `3 : three' ... ");
	insert(my_heap, 3, "three");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `50 : fifty' ... ");
	insert(my_heap, 50, "fifty");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `11 : eleven' ... ");
	insert(my_heap, 11, "eleven");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("========== Extractions ==========\n\n");

	/** Retrieve the min element. **/
	extract_min(my_heap);
	extract_min(my_heap);
	extract_min(my_heap);
	extract_min(my_heap);
	extract_min(my_heap);
	extract_min(my_heap);
	extract_min(my_heap);
	extract_min(my_heap);

	printf("========== Mixed Insertions & Extractions ==========\n\n");

	printf("Insert `22 : twenty two' ... ");
	insert(my_heap, 22, "twenty two");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `6 : six' ... ");
	insert(my_heap, 6, "six");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `73 : seventy three' ... ");
	insert(my_heap, 73, "seventy three");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	extract_min(my_heap);
	extract_min(my_heap);

	printf("Insert `21 : twenty one' ... ");
	insert(my_heap, 21, "twenty one");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	extract_min(my_heap);

	printf("Insert `10 : ten' ... ");
	insert(my_heap, 10, "ten");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `54 : fifty four' ... ");
	insert(my_heap, 54, "fifty four");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	printf("Insert `72 : seventy two' ... ");
	insert(my_heap, 72, "seventy two");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	extract_min(my_heap);
	extract_min(my_heap);
	extract_min(my_heap);

	printf("Insert `99 : ninty nine' ... ");
	insert(my_heap, 99, "ninty nine");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	extract_min(my_heap);

	printf("Insert `13 : thirdteen' ... ");
	insert(my_heap, 13, "thirdteen");
	printf("done.\n");
	get_size(my_heap);
	print_heap(my_heap);

	extract_min(my_heap);
	extract_min(my_heap);

	/** Destroy min-heap. **/
	printf("Destroy the min-heap ... ");
	minheap_destroy(my_heap);
	printf("done.\n");

	return 0;
}

/** Perform insertion. **/
int insert(minheap_t *h, long int key_val, const char *data)
{
	mh_key_t *key = NULL;
	char *buf = (char *)malloc(sizeof(char)*(strlen(data)+1));
	strncpy(buf, data, strlen(data));
	buf[strlen(data)] = '\0';

	key = gen_key(key_val);

	if (minheap_insert(h, key, buf))
	{
		return -1;
	}

	return 0;
}

/** Generates a proper key. **/
mh_key_t *gen_key(long int i)
{
	mh_key_t *k = (mh_key_t *)malloc(sizeof(mh_key_t));
	k->key = i;
	return k;
}

/** Get some stats on the min-heap. **/
void get_size(minheap_t *h)
{
	printf("   Heap size : s_size: %d : %d\n", (int)minheap_size(h), (int)minheap_s_size(h));
}

/** Extract minimum element. **/
void extract_min(minheap_t *h)
{
	char *buf = (char *)minheap_extract_min(h);
	printf("Extract-Min: %s\n", buf);
	get_size(h);
	print_heap(h);
	free(buf);
}

/** Outputs the heap. **/
void print_heap(minheap_t *h)
{
	printf("   ");
	minheap_print(h);
	printf("\n");
}
